package GGI::Pixel;

=head1 NAME

GGI::Pixel - Perl extension implementing an interface to LibGGI Get/Put buffers

=head1 SYNOPSIS

  use GGI::Pixel;

  $pixels = new GGI::Pixel;
  $pixels = new GGI::Pixel $visual $numpixels;

  $bytes_used_per_pixel = $pixels->size;

  $owner = $pixels->lastused;
  $pixels->lastused($visual);

=head1 DESCRIPTION

The GGI::Pixel object encapsulates raw memory buffers used with
GGI:: objects to copy and manipulate graphic image data.

=cut

use strict;
use Carp;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK $AUTOLOAD);

require Exporter;
require DynaLoader;
require AutoLoader;


=head1 Exported constants

  ATTR_BGCOLOR
  ATTR_BLINK
  ATTR_BOLD
  ATTR_BRIGHT
  ATTR_FGCOLOR
  ATTR_FONT
  ATTR_HALF
  ATTR_INTENSITY
  ATTR_ITALIC
  ATTR_NORMAL
  ATTR_REVERSE
  ATTR_UNDERLINE
  GGI_BM_SUB_ALPHA
  GGI_BM_SUB_BGCOL
  GGI_BM_SUB_BLINK
  GGI_BM_SUB_BLUE
  GGI_BM_SUB_BOLD
  GGI_BM_SUB_CLUT
  GGI_BM_SUB_CYAN
  GGI_BM_SUB_FGCOL
  GGI_BM_SUB_FONTSEL
  GGI_BM_SUB_GREEN
  GGI_BM_SUB_INTENSITY
  GGI_BM_SUB_ITALIC
  GGI_BM_SUB_K
  GGI_BM_SUB_MAGENTA
  GGI_BM_SUB_MODESEL
  GGI_BM_SUB_PALSEL
  GGI_BM_SUB_RED
  GGI_BM_SUB_TEXNUM
  GGI_BM_SUB_U
  GGI_BM_SUB_UNDERLINE
  GGI_BM_SUB_V
  GGI_BM_SUB_WINDOWID
  GGI_BM_SUB_WRITEPROT
  GGI_BM_SUB_Y
  GGI_BM_SUB_YELLOW
  GGI_BM_SUB_ZBUFFER
  GGI_BM_TYPE_ATTRIB
  GGI_BM_TYPE_COLOR
  GGI_BM_TYPE_LOGIC
  GGI_BM_TYPE_NONE
  GGI_COLOR_PRECISION
  GGI_PALETTE_DONTCARE
  GGI_PF_EXTENDED
  GGI_PF_HAM
  GGI_PF_HIGHBIT_RIGHT
  GGI_PF_REVERSE_ENDIAN

=cut

@ISA = qw(Exporter DynaLoader);
# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.
@EXPORT = qw(
	ATTR_BGCOLOR
	ATTR_BLINK
	ATTR_BOLD
	ATTR_BRIGHT
	ATTR_FGCOLOR
	ATTR_FONT
	ATTR_HALF
	ATTR_INTENSITY
	ATTR_ITALIC
	ATTR_NORMAL
	ATTR_REVERSE
	ATTR_UNDERLINE
	GGI_BM_SUB_ALPHA
	GGI_BM_SUB_BGCOL
	GGI_BM_SUB_BLINK
	GGI_BM_SUB_BLUE
	GGI_BM_SUB_BOLD
	GGI_BM_SUB_CLUT
	GGI_BM_SUB_CYAN
	GGI_BM_SUB_FGCOL
	GGI_BM_SUB_FONTSEL
	GGI_BM_SUB_GREEN
	GGI_BM_SUB_INTENSITY
	GGI_BM_SUB_ITALIC
	GGI_BM_SUB_K
	GGI_BM_SUB_MAGENTA
	GGI_BM_SUB_MODESEL
	GGI_BM_SUB_PALSEL
	GGI_BM_SUB_RED
	GGI_BM_SUB_TEXNUM
	GGI_BM_SUB_U
	GGI_BM_SUB_UNDERLINE
	GGI_BM_SUB_V
	GGI_BM_SUB_WINDOWID
	GGI_BM_SUB_WRITEPROT
	GGI_BM_SUB_Y
	GGI_BM_SUB_YELLOW
	GGI_BM_SUB_ZBUFFER
	GGI_BM_TYPE_ATTRIB
	GGI_BM_TYPE_COLOR
	GGI_BM_TYPE_LOGIC
	GGI_BM_TYPE_NONE
	GGI_COLOR_PRECISION
	GGI_PALETTE_DONTCARE
	GGI_PF_EXTENDED
	GGI_PF_HAM
	GGI_PF_HIGHBIT_RIGHT
	GGI_PF_REVERSE_ENDIAN
);
$VERSION = '0.01';


=head1 Methods

The methods applicable to GGI::Pixel objects are as follows:

=head2 new

The GGI::Pixel constructor can be called with no arguments to yeild
a GGI::Pixel object with no associated memory buffer.  Later passing
this object to a LibGGI drawing operation will cause the buffer to 
be auto-allocated to the appropriate size, and associated with the
visual in question.

Optionally, a GGI:: object can be passed as a parameter to the constructor, 
causing the GGI::Pixel object to be associated with it without performing 
a LibGGI operation.  If this is done, an additional integer parameter may be
provided to pre-allocate a buffer representing that number of pixels.

=cut

sub new {
    my ($class) = shift();
    my $this = {};
    $this->{grow_put} = 1;
    $this->{grow_get} = 1;
    $this->{shrink_put} = 0;
    $this->{shrink_get} = 0;
    $this->{numpix} = undef;
    $this->{obj} = undef;
    $this->{lastused} = shift();
    bless $this, $class;
    $this->_malloc(shift()) if (@_ + 0);
    return $this;
}

=head2 lastused

The lastused method associates a GGI::Pixel object with the specified
GGI:: object, or simply returns the GGI:: object it is associated with 
if it is given no arguments.  This is usually done automatically when the 
GGI::Pixel object is used in a GGI:: object method, so unless you are using 
the advanced buffer management features described below there is 
little need to call it.

=cut

sub lastused {
    my ($this) = shift();
    return $this->{lastused} unless (@_ + 0);
    if(defined($this->{lastused})) {
	$this->{numpix} *= $this->size;
	$this->{lastused} = shift();
	$this->{numpix} /= $this->size;
    }
    else {
	$this->{lastused} = shift();
    }
    return $this->{lastused};
}

=head1 size

Gets the size in bytes of buffered pixels based on the GGI:: object that
was last associated with this GGI::Pixel object; returns undef if the
GGI::Pixel object has yet to be associated with a GGI:: object

=cut

sub size {
    my ($this) = shift();
    return undef unless defined($this->{lastused});
    return ggi_get_pixel_size($this->{lastused}{obj});
}

sub resize_put {
    my($this) = shift();
    my($numpix) = shift();
    unless ($this->{obj}) {
	$this->{numpix} = $numpix;
	$this->{obj} = $this->_malloc($numpix) if (@_ + 0);
	unless ($this->{obj}) {
	    $this->{numpix} = 0 ;
	    $this->{obj} = undef;
	    return 0;
	}
	return 1;
    }
    if (($numpix > $this->{numpix} and $this->{grow_put}) 
	or ($numpix < $this->{numpix} and $this->{shrink_put})) {
	$this->{obj} = ggi_realloc_pixels($this->{lastused}{obj},
					  $this->{obj}, 
					  $numpix,
					  $this->{numpix});
	return 0 if $this->{numpix} == $numpix;
	$this->{numpix} = $numpix;
    }
    return 1;
}

sub resize_get {
    my($this) = shift();
    my($numpix) = shift();
    unless ($this->{obj}) {
	$this->{numpix} = $numpix;
	$this->{obj} = $this->_malloc($numpix);
	unless ($this->{obj}) {
	    $this->{numpix} = 0 ;
	    $this->{obj} = undef;
	    return 0;
	}
	return 1;
    }
    if (($numpix > $this->{numpix} and $this->{grow_get})
	or ($numpix < $this->{numpix} and $this->{shrink_get})) {
	$this->{obj} = ggi_realloc_pixels($this->{lastused}{obj},
					  $this->{obj}, 
					  $numpix,
					  $this->{numpix});
	return 0 if $this->{numpix} == $numpix;
	$this->{numpix} = $numpix;       
    }
    return 1;
}


sub _malloc {
    my ($this) = shift();
    $this->{numpix} = int(shift());
    $this->{obj} = ggi_malloc_pixels($this->{lastused}{obj}, 
				     $this->{numpix});
    unless ($this->{obj}) {
	$this->{obj} = undef;
	return undef;
    }
    $this->{did_alloc} = 1;
    return ($this->{obj});
}

sub DESTROY {
    my ($this) = shift();
    ggi_free_pixels($this->{lastused}{obj})
	if $this->{did_alloc} ;
}

sub AUTOLOAD {
    my $constname;
    ($constname = $AUTOLOAD) =~ s/.*:://;
    my $val = constant($constname, @_ ? $_[0] : 0);
    if ($! != 0) {
	if ($! =~ /Invalid/) {
	    $AutoLoader::AUTOLOAD = $AUTOLOAD;
	    goto &AutoLoader::AUTOLOAD;
	}
	else {
		croak "Your vendor has not defined GGI::Pixel macro $constname";
	}
    }
    eval "sub $AUTOLOAD { $val }";
    goto &$AUTOLOAD;
}

bootstrap GGI::Pixel $VERSION;

# Preloaded methods 

1;
__END__

=head1 AUTHOR

Brian S. Julin <bri@calyx.com>

=head1 SEE ALSO

GGI::(3pm), perl(1).

=cut
